"""Default configuration module for use with gunicorn.

Usage:

    gunicorn -c python:impact_stack.observability.tracing.gunicorn wsgi:app

In your app's pyproject.toml you need to define the following Python entry point
to refer to the tracing extension instance (this then get's picked up by this
gunicorn hook):

    [project.entry-points.impact_stack_observability_tracing]
    gunicorn = "myapp.tracing:tracing"

If you want to or need to use a config file like gunicorn.conf.py, you can just
import the `post_fork` name into your config file to get the default behaviour:

    # gunicorn.conf.py

    from impact_stack.observability.tracing.gunicorn import post_fork

    # other hooks
    def pre_fork(*args):
        pass

If you want to extent a post_hook you can call the hook provided in this module
in your own hook:

    # gunicorn.conf.py

    from impact_stack.observability.tracing.gunicorn import post_fork_tracing

    def post_fork(*args):
        app = server.app.wsgi()

        # Initialize tracing
        post_fork_tracing(app, {})

        # Additional behaviour
        # ...

"""

import logging

from . import utils


def post_fork_tracing(app, resource_attributes):  # pragma: no cover
    """Utility method to start tracing.

    You can call this from your custom post_fork hook as well.
    """
    # Use gunicorn logger as we do not require the WSGI app to be a Flask app
    logger = logging.getLogger("gunicorn.error")
    logger.debug("Initializing tracing in gunicorn worker")

    tracing_ext = utils.load_one_entry_point_by_name_or_none(
        "impact_stack_observability_tracing", "gunicorn"
    )

    # Initialize tracing. This starts emiting tracing signals.
    if tracing_ext:
        tracing_ext.init_app(app, resource_attributes)


def post_fork(server, _worker):  # pragma: no cover
    """Gunicorn post fork hook for initializing tracing.

    Gunicorn works with a fork process model and thus a special treatment is needed when
    initializing the tracer. See
    https://opentelemetry-python.readthedocs.io/en/latest/examples/fork-process-model/README.html
    """
    app = server.app.wsgi()
    resource_attributes = {"wsgi_server": "gunicorn"}
    post_fork_tracing(app, resource_attributes)
